# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

from tvtk.tvtk_classes.scalar_tree import ScalarTree


class SpanSpace(ScalarTree):
    """
    SpanSpace - organize data according to scalar span space
    
    Superclass: ScalarTree
    
    This is a helper class used to accelerate contouring operations.
    Given an dataset, it organizes the dataset cells into a 2d binned
    space, with axes (scalar_min,scalar_max). This so-called span space
    can then be traversed quickly to find the cells that intersect a
    particular contour value.
    
    This class has an API that supports both serial and parallel
    operation.  The parallel API enables the using class to grab arrays
    (or batches) of cells that lie along a particular row in the span
    space. These arrays can then be processed separately in parallel.
    
    Learn more about span space in these two publications: 1) "A Near
    Optimal Isosorface Extraction Algorithm Using the Spsn Space." Yarden
    Livnat et al. and 2) Isosurfacing in SPan Space with Utmost
    Efficiency." Han-Wei Shen et al.
    
    @sa
    SimpleScalarTree
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkSpanSpace, obj, update, **traits)
    
    resolution = traits.Trait(100, traits.Range(1, 2147483647, enter_set=True, auto_set=False), help=\
        """
        Set/Get the resolution N of the span space. The span space can be
        envisioned as a rectangular lattice of NXN buckets (i.e., N rows
        and N columns), where each bucket stores a list of cell ids. The
        i-j coordinate of each cell (hence its location in the lattice)
        is determined from the cell's 2-tuple (smin,smax) scalar range.
        By default Resolution = 100.
        """
    )

    def _resolution_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetResolution,
                        self.resolution)

    _updateable_traits_ = \
    (('global_warning_display', 'GetGlobalWarningDisplay'),
    ('reference_count', 'GetReferenceCount'), ('debug', 'GetDebug'),
    ('resolution', 'GetResolution'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'resolution'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(SpanSpace, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit SpanSpace properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['resolution']),
            title='Edit SpanSpace properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit SpanSpace properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

